from distutils.core import setup
setup(
  name = 'ionmf',
  packages = ['ionmf', 'ionmf.factorization'],
  version = '0.3.5',
  description = 'Integrative orthogonal non-negative matrix factorization with examples.',
  author = 'Martin Strazar',
  author_email = 'martin.strazar@gmail.com',
  url = 'https://github.com/mstrazar/iONMF',
  download_url = 'https://github.com/mstrazar/iONMF/tarball/0.3.4',
  keywords = ['data integration', 'matrix factorization', 'machine learning'],
  classifiers = [],
)
